/*
Der Original-Code von Peter Dannegger auf einen ATTiny25/45/85 portiert.
Slave-Rolle. Es wird der Pin PB4 als i/o genutzt.

Timer1 wird sowohl zum Senden als auch für den Empfang genutzt.
Senden:  OCR1B toggelt den Sendepin PB4, das gleichgeschaltet OCR1C loescht den Zaehler bei COMPARE
         OCR1A wird beim Senden nicht benoetigt.
Empfang: OCR1A misst die 4T Zeit (Synch-Bit), OCR1B	die 2T = "1" Zeit. Erste fallende Flanke startet
         den Empfang und initialisiert den Empfangsbuffer. Jede weitere Flanke triggert den Timer1 neu.
		 Sollten Flanken ausbleiben oder mehr als erwartet empfangen werden, wird das Paket verworfen.
		 Das geschiet ueber ISR TIMER1_COMPA.
*/

ISR( TIMER1_COMPB_vect )        // Transmit interrupt
{
uint8_t *dp;					// locals for faster access

if( tx_idx < DATA_SIZE * 8 ){
	OCR1B = BIT_TIME * 1 - 0.5;     // 0-bit: 1T
	dp = &tx_buff[tx_idx++ / 8];    // get byte address
	if( *dp & 0x80 )                // msb first
		OCR1B = BIT_TIME * 2 - 0.5; // 1-bit: 2T
	*dp <<= 1;                      // shift bits
}else{
	OCR1B = BIT_TIME * 4 - 0.5;     // sync:  4T
	if (PINB & (1<<PB4) )           // after output = high
		{
		TCCR1 = 0;                  // then stop T1
		}
	}
OCR1C = OCR1B;						// CTC
}


void tx_data( uint8_t addr, uint8_t data )
{
	PCMSK = 0;    				// disable receive
	TCCR1 = 0;					// stop T1
	tx_idx = 0;                 // point to first bit
	tx_buff[0] = addr;
	tx_buff[1] = data;
	tx_buff[2] = Slow_CRC_Cal8Bits(0x00, 2, tx_buff);
	DDRB &= ~(1<<PB4);          // input
	PORTB |= (1<<PB4);                // pullup on
	while( (PINB & (1<<PB4)) == 0 );  // wait until pin high
	TIFR = 1<<OCF1B;                  // clear earlier interrupts
	TCNT1 = 0;
	OCR1B = BIT_TIME * 4 - 0.5;       // sync: 4T
	OCR1C = OCR1B;					  // OCR1C besorgt das CTC des TCNT1
	GTCCR = 1<<COM1B1 | 1<<COM1B0 | 1<<FOC1B; // set on compare output high
	GTCCR = 1<<COM1B0;			//TCCR1A = 1<<COM1B0;// toggle pin on compare
	DDRB |= (1<<PB4);          // tx output on
//	nop();  nop();             // 2 cycle to clear flags !
	asm volatile ("nop");
	asm volatile ("nop");
	TIMSK |= 1<<OCIE1B;                 // enable tx interrupt
	TCCR1 = (1<<CTC1) | PRESCALER_SET;  // start T1, CTC Mode
//	while( TCCR1 );                     // until tx done
}


/****************************** Receiving *******************************/

void rx_init( void )
{
	TCCR1 = 0;						// Stop T0
	rx_idx = 255;                   // detect sync transition
	DDRB &= ~(1<<PB4);              // input
	PORTB |= (1<<PB4);              // pullup on				
	GTCCR =0;						// disconnect tx output
	TIFR = 1<<OCF1A | 1<<OCF1B;     // clear earlier interrupts
	TCNT1 = 0;
	OCR1B = BIT_TIME * 1.4 - 0.5;   // detect 2T
	OCR1A = BIT_TIME * 2.8 - 0.5;   // detect 4T
//	TIMSK |= 1<<OCIE1A;             // sync interrupt Denke der IR-EN ist hier zu frueh freigeschaltet
	TIMSK &= ~((1<<OCIE1A) | (1<<OCIE1B));
	TCCR1 = PRESCALER_SET;          // start Timer
	GIFR = 1<<PCIF;                 // clear earlier interrupts             
	PCMSK |= (1<< PCINT4);			// pin change interrupt 
//	PB4state = (PINB & (1<<PB4));	// Startwert, erlaubt Filterung des Gruppen-IRQ auf einen Pin
}

ISR( TIMER1_COMPA_vect )          // sync received
{
	if( rx_idx == DATA_SIZE * 8 ) // receive successful
		{
//	PCMSK1 = 0;					// disable further receive
		PCMSK &= ~(1<< PCINT4);
		}
	rx_idx = 255;               // ready for start transition
	TIMSK &= ~(1<<OCIE1A);		// weitere Sync-Suche abschalten
}

ISR (PCINT0_vect)
{ 
/*
Code für andere PIN-Interrupts
*/

#define rx_idx  tx_idx
#define rx_buff tx_buff

if (PB4state != (PINB & (1<<PB4)))  // Receive interrupt
{
	PB4state = (PINB & (1<<PB4));
	uint8_t *dp;					// locals for faster access
	uint8_t i;						// locals for faster access	
	TCNT1 = 0;
	if( rx_idx >= DATA_SIZE * 8 ){      // on start or stop bit
		rx_idx = 0;                     // point to first bit
	}else{
		dp = &rx_buff[rx_idx / 8];      // get byte address
		i = *dp << 1;                   // shift bits
		if( TIFR & 1<<OCF1B )           // if pulse > 1.4T
			i++;                        // msb first
		*dp = i;
		rx_idx++;                       // point to next bit
		}
	TIFR = (1<<OCF1B) | (1<<OCF1A);     // clear 2T AND 4T detection flag
	TIMSK |= 1<<OCIE1A;					// erst ab hier auf 4T testen 
	} 
}